"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UPDATE_REQUEST_COMMENTS = exports.GET_TENANT_DETAILS_BY_ID = exports.GET_ALL_SERVICE_APPS = exports.UPDATE_APP_ATTRIBUTES = exports.GET_APP_DETAILS_BY_NAME = exports.PALADIN_ADMINS = exports.SUBJECT_NAME = exports.APPROVER = exports.ALL_DEVELOPERS_APP = exports.ALL_APPROVERS = exports.REQUESTED_APP = exports.REQUEST_INFO = void 0;
const apollo_boost_1 = require("apollo-boost");
exports.REQUEST_INFO = (0, apollo_boost_1.gql) `
  query requestInfo($requestId: uuid!) {
    request(where: { id: { _eq: $requestId } }) {
      id
      type
      status
      rquestedBy {
        id
        user_name
        display_name
        email_id
      }
      approvedBy {
        id
        display_name
      }
      requested_data
      comments
    }
  }
`;
exports.REQUESTED_APP = (0, apollo_boost_1.gql) `
  query requestedApp($resourceId: uuid!) {
    app_product(where: { id: { _eq: $resourceId } }) {
      app {
        id
        display_name
        attributes
      }
      environment {
        name
      }
    }
  }
`;
exports.ALL_APPROVERS = (0, apollo_boost_1.gql) `
  query allApprovers($appId: uuid!) {
    app(where: { id: { _eq: $appId } }) {
      groups(where: { kind: { _eq: "approver" } }) {
        group {
          id
          users {
            user {
              user_name
              display_name
            }
          }
        }
      }
    }
  }
`;
exports.ALL_DEVELOPERS_APP = (0, apollo_boost_1.gql) `
  query allApprovers($appName: String!) {
    app(where: { name: { _eq: $appName } }) {
      id
      display_name
      attributes
      groups(where: { kind: { _eq: "developer" } }) {
        group {
          id
          users {
            user {
              user_name
            }
          }
        }
      }
    }
  }
`;
exports.APPROVER = (0, apollo_boost_1.gql) `
  query userInfo($approverId: uuid!) {
    user(where: { id: { _eq: $approverId } }) {
      user_name
      display_name
    }
  }
`;
exports.SUBJECT_NAME = (0, apollo_boost_1.gql) `
  query subjectName($subjectId: uuid!) {
    user(where: { id: { _eq: $subjectId } }) {
      display_name
      user_name
    }
  }
`;
exports.PALADIN_ADMINS = (0, apollo_boost_1.gql) `
  query PaladinAdmin {
    group(where: { name: { _eq: "paladin-admins" } }) {
      users {
        user {
          display_name
          user_name
        }
      }
    }
  }
`;
exports.GET_APP_DETAILS_BY_NAME = (0, apollo_boost_1.gql) `
  query getAppDetailsByName($appName: String) {
    app(where: { name: { _eq: $appName } }) {
      id
      name
      attributes
      tenant_id
    }
  }
`;
exports.UPDATE_APP_ATTRIBUTES = (0, apollo_boost_1.gql) `
mutation updateAppAttribute($appId: uuid!, $attributes: jsonb!) {
  update_app(where: {id: {_eq: $appId}}, _set: {attributes: $attributes}) {
    affected_rows
  }
}
`;
exports.GET_ALL_SERVICE_APPS = (0, apollo_boost_1.gql) `
  query getServiceApps {
    app(where: { attributes: { _contains: [{ name: "app-type", value: "service" }] } }) {
      id
      name
      display_name
      attributes
    }
  }
`;
exports.GET_TENANT_DETAILS_BY_ID = (0, apollo_boost_1.gql) `
  query getTenantInfo($tenantId: uuid!) {
    tenant(where: { id: { _eq: $tenantId } }) {
      name
      attributes
    }
  }
`;
exports.UPDATE_REQUEST_COMMENTS = (0, apollo_boost_1.gql) `
mutation RequestEdit($requestId: uuid!, $requestUpdateInput: jsonb!) {
  update_request(where: { id: { _eq: $requestId } }, _set: {comments: $requestUpdateInput}) {
    affected_rows
  }
}
`;
