"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApigeeApiService = void 0;
const lambda_logger_1 = require("../lib/lambda-logger");
const axios = require("axios");
const qs_1 = __importDefault(require("qs"));
var fs = require("fs");
var unzipper = require("unzipper");
var replace = require("replace-in-file");
var zipLib = require("zip-lib");
const request = require("request-promise");
const FormData = require("form-data");
const aws_sdk_1 = require("aws-sdk");
class ApigeeApiService {
    constructor() {
        this.accessToken = "";
        this.apigeeAccessTokenUrl = "https://hp.login.apigee.com/oauth/token";
        this.apigeeAccessTokenUrlResetKey = "https://hp.login.apigee.com/oauth/token";
        this.apigeeBaseUrl = "https://api.enterprise.apigee.com/v1/organizations/hpcorp-ob/";
        this.apigeeSecretResetUrl = "https://api.enterprise.apigee.com/v1/organizations/hpcorp-ob/developers/MPS-RCB-AUTH@HP.onmicrosoft.com/apps/";
        this.log = lambda_logger_1.LambdaLogger.getInstance("ApigeeApiService");
        if (process.env.stage === "prod") {
            this.apigeeAccessTokenUrlResetKey =
                "https://core.api.hp.com/clientServices/v1/token";
            this.apigeeBaseUrl =
                "https://api.enterprise.apigee.com/v1/organizations/hpcorp/";
            this.apigeeSecretResetUrl =
                "https://core.api.hp.com/clientServices/v1/clients/clientIdPlaceHolder/secret";
        }
    }
    generateAccessToken(username, password, isResetKeyOperation) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Getting token from apigee server");
            var url = this.apigeeAccessTokenUrl;
            if (isResetKeyOperation) {
                url = this.apigeeAccessTokenUrlResetKey;
            }
            let headers = {};
            let formData = {};
            if (process.env.stage !== "prod") {
                headers = {
                    Authorization: "Basic ZWRnZWNsaTplZGdlY2xpc2VjcmV0",
                    Accept: "application/json;charset=utf-8",
                    "content-type": "application/x-www-form-urlencoded",
                };
                formData = {
                    username: username,
                    password: password,
                    grant_type: "password",
                    response_type: "access_token",
                };
            }
            else {
                if (isResetKeyOperation) {
                    const basicAuthToken = Buffer.from(`${username}:${password}`, "utf8").toString("base64");
                    headers = {
                        "content-type": "application/x-www-form-urlencoded",
                        Authorization: `Basic ${basicAuthToken}`,
                    };
                    formData = {
                        grant_type: "client_credentials",
                    };
                }
                else {
                    headers = {
                        Authorization: "Basic ZWRnZWNsaTplZGdlY2xpc2VjcmV0",
                        Accept: "application/json;charset=utf-8",
                        "content-type": "application/x-www-form-urlencoded",
                    };
                    formData = {
                        username: username,
                        password: password,
                        grant_type: "password",
                        response_type: "access_token",
                    };
                }
            }
            const options = {
                method: "POST",
                headers: headers,
                data: qs_1.default.stringify(formData),
                url: url,
            };
            let response = {};
            try {
                response = yield axios(options);
                if (response && response.data) {
                    this.log.info("Got the token from apigee");
                    this.accessToken = response.data.access_token;
                }
            }
            catch (error) {
                this.log.error("Error while getting apigee token: " + error);
                response.result = false;
            }
            return response;
        });
    }
    deleteCurrentClienId(appName, clientId) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeSecretResetUrl + appName + "/keys/" + clientId;
            let response = yield this.makeApigeeApiRequest(url, "delete", "");
            return response;
        });
    }
    createNewClientId(appName, clientId, newSecret) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeSecretResetUrl + appName + "/keys/create";
            let body = {
                consumerKey: clientId,
                consumerSecret: newSecret,
            };
            let response = yield this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    addApiProductsToClientId(appName, clientId, products) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeSecretResetUrl + appName + "/keys/" + clientId;
            let body = {
                apiProducts: products,
            };
            let response = yield this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    updateAppAttribute(appName, attributeName, attributeValue) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeSecretResetUrl + appName + "/attributes/" + attributeName;
            let body = {
                value: attributeValue,
            };
            let response = yield this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    resetClientSecret(clienId, encryptedSecret) {
        return __awaiter(this, void 0, void 0, function* () {
            let headers = {
                Authorization: "Bearer " + this.accessToken,
                encryptedSecret: encryptedSecret,
            };
            this.apigeeSecretResetUrl = this.apigeeSecretResetUrl.replace("clientIdPlaceHolder", clienId);
            let response = yield this.makeApigeeApiRequest(this.apigeeSecretResetUrl, "POST", "", headers);
            return response;
        });
    }
    createTargetServer(serverName, targetHostname, env) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl + "environments/" + env + "/targetservers";
            var body = {
                host: targetHostname,
                isEnabled: true,
                name: serverName,
                port: 443,
                sSLInfo: {
                    ciphers: [],
                    clientAuthEnabled: "false",
                    enabled: "true",
                    ignoreValidationErrors: false,
                    protocols: [],
                    trustStore: "",
                },
            };
            var response = yield this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    updateTargetServer(serverName, targetHostname, env) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/targetservers/" +
                serverName;
            var body = {
                host: targetHostname,
                isEnabled: true,
                port: 443,
                sSLInfo: {
                    ciphers: [],
                    clientAuthEnabled: "false",
                    enabled: "true",
                    ignoreValidationErrors: false,
                    protocols: [],
                    trustStore: "",
                },
            };
            var response = yield this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    getProxyDetails(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl + "apis/" + name;
            var response = yield this.makeApigeeApiRequest(url, "GET", "");
            return response;
        });
    }
    deployProxy(name, env, revision) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/apis/" +
                name +
                "/revisions/" +
                revision +
                "/deployments?override=true";
            var response = yield this.makeApigeeApiRequest(url, "POST", "");
            return response;
        });
    }
    getProductDetails(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl + "apiproducts/" + name;
            var response = yield this.makeApigeeApiRequest(url, "GET", "");
            return response;
        });
    }
    getStatistics(select, timeRange, timeUnit, proxyName, env) {
        return __awaiter(this, void 0, void 0, function* () {
            let queryParam = "select=" + select + "&timeRange=" + timeRange + "&timeUnit=" + timeUnit;
            // let queryParam: string =
            //   "select=sum(message_count)&timeRange=2/8/2021%2000:00~2/10/2021%2023:59&timeUnit=day";
            let url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/stats/apiproxy/" +
                proxyName +
                "?" +
                queryParam;
            var response = this.makeApigeeApiRequest(url, "GET", "");
            return response;
        });
    }
    deployProduct(name, payload, env) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl + "apiproducts/" + name;
            var body = {};
            body.approvalType = "auto";
            body.attributes = payload.attributes;
            body.name = name;
            body.displayName = payload.displayName;
            body.description = payload.description;
            body.environments = payload.environments;
            body.environments.push(env);
            body.proxies = payload.proxies;
            var response = this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    updateProduct(name, payload) {
        let url = this.apigeeBaseUrl + "apiproducts/" + name;
        var body = {};
        body.approvalType = "auto";
        body.attributes = payload.attributes;
        body.name = name;
        body.displayName = payload.displayName;
        body.description = payload.description;
        body.environments = payload.environments;
        body.proxies = payload.proxies;
        var response = this.makeApigeeApiRequest(url, "POST", body);
        return response;
    }
    createProduct(displayName, shortName, description, env, attributes) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeBaseUrl + "apiproducts";
            var body = {
                approvalType: "auto",
                environments: [],
                proxies: [],
            };
            body.displayName = displayName;
            body.name = shortName;
            body.description = description;
            body.attributes = attributes;
            body.attributes.push({
                name: "short-name",
                value: shortName,
            });
            body.environments.push(env);
            body.proxies.push(shortName + "-proxy");
            var response = this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    createApiProxyPackage(productName) {
        return __awaiter(this, void 0, void 0, function* () {
            let s3 = new aws_sdk_1.S3();
            const params = {
                Bucket: "paladin-tropos-apigee-proxy-template",
                Key: "proxy_template_tropos.zip",
            };
            var data = yield s3.getObject(params).promise();
            fs.writeFileSync("/tmp/proxy_template_tropos.zip", data.Body);
            let folderPath = "/tmp/";
            var proxyZipFileName = folderPath + productName + "-proxy.zip";
            if (process.env.stage === "prod") {
                var proxyZipFileName = folderPath + "hpcorp-ob-" + productName + "-proxy.zip";
            }
            return new Promise(function (resolve) {
                fs.createReadStream(folderPath + "proxy_template_tropos.zip").pipe(unzipper.Extract({ path: folderPath }).on("close", function () {
                    var apiProxyFolderpath = folderPath + "proxy_template/apiproxy/";
                    fs.renameSync(apiProxyFolderpath + "proxy_template.xml", apiProxyFolderpath + productName + "-proxy.xml");
                    var replaceOptions = {
                        files: apiProxyFolderpath + productName + "-proxy.xml",
                        from: ["proxy_base_path", "proxy_display_name"],
                        to: [productName, productName],
                    };
                    replace.sync(replaceOptions);
                    replaceOptions = {
                        files: apiProxyFolderpath + "targets/default.xml",
                        from: ["server_name"],
                        to: [productName + "-server"],
                    };
                    replace.sync(replaceOptions);
                    replaceOptions = {
                        files: apiProxyFolderpath + "proxies/default.xml",
                        from: ["proxy_base_path"],
                        to: [productName],
                    };
                    replace.sync(replaceOptions);
                    zipLib
                        .archiveFolder(folderPath + "proxy_template", proxyZipFileName)
                        .then(function () {
                        resolve({
                            result: "success",
                        });
                    });
                }));
            });
        });
    }
    importApiProxy(shortName) {
        return __awaiter(this, void 0, void 0, function* () {
            let proxyName = shortName + "-proxy";
            let folderPath = "/tmp/";
            var proxyZipFile = folderPath + shortName + "-proxy.zip";
            var url = this.apigeeBaseUrl + "/apis?action=import&name=" + proxyName;
            var formData = new FormData();
            formData.append("file", fs.createReadStream(proxyZipFile));
            let response = yield axios.post(url, formData, {
                headers: {
                    Authorization: "Bearer " + this.accessToken,
                    "Content-Type": "multipart/form-data",
                },
            });
            fs.unlinkSync(folderPath + shortName + "-proxy.zip");
            return response.data;
        });
    }
    getAppDetails(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = this.apigeeSecretResetUrl + name;
            var response = yield this.makeApigeeApiRequest(url, "GET", "");
            return response;
        });
    }
    createApp(displayName, shortName) {
        return __awaiter(this, void 0, void 0, function* () {
            var body = {
                attributes: [],
            };
            body.name = shortName;
            body.attributes.push({
                name: "DisplayName",
                value: displayName,
            });
            var currentDateInSeconds = Math.round(Date.now() / 1000);
            body.attributes.push({
                name: "secretCreatedDate",
                value: currentDateInSeconds,
            });
            var oneDayTimeSpanInSeconds = 24 * 60 * 60;
            var secretExpiryDate = currentDateInSeconds + 90 * oneDayTimeSpanInSeconds;
            body.attributes.push({
                name: "secretExpiryDate",
                value: secretExpiryDate,
            });
            body.attributes.push({
                name: "oldSecret",
                value: "",
            });
            var response = this.makeApigeeApiRequest(this.apigeeSecretResetUrl, "POST", body);
            return response;
        });
    }
    AddKvmEntry(name, appType, secretRotationOptIn, secretPath, clientId, env, tenantId, tokenExpiryTime) {
        return __awaiter(this, void 0, void 0, function* () {
            var url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries";
            var payload = {
                webAppTypeValue: '{"hpid-auth":"","hponeid-auth":"","p-tenant-id":"tenantIdPlaceHolder","consumerKey":"consumerkeyPlaceHolder"}',
                serviceTypeValue: '{"p-tenant-id":"tenantIdPlaceHolder","consumerKey":"consumerkeyPlaceHolder","secretRotationOptIn":secretRotationOptInPlaceHolder,"secretPath":"secretPathPlaceHolder","tokenExpiryTime":"tokenExpiryTimePlaceHolder"}',
            };
            var data;
            if (appType === "web") {
                data = payload.webAppTypeValue;
                data = data.replace("consumerkeyPlaceHolder", clientId);
                data = data.replace("tenantIdPlaceHolder", tenantId);
            }
            else {
                data = payload.serviceTypeValue;
                data = data.replace("consumerkeyPlaceHolder", clientId);
                data = data.replace("secretRotationOptInPlaceHolder", secretRotationOptIn);
                data = data.replace("secretPathPlaceHolder", secretPath);
                data = data.replace("tenantIdPlaceHolder", tenantId);
                data = data.replace("tokenExpiryTimePlaceHolder", tokenExpiryTime);
            }
            var body = {
                name: name,
                value: data,
            };
            var response = this.makeApigeeApiRequest(url, "POST", body);
            return response;
        });
    }
    DeleteKvmEntry(name, env) {
        return __awaiter(this, void 0, void 0, function* () {
            var url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries/" +
                name;
            var response = this.makeApigeeApiRequest(url, "DELETE", "");
            return response;
        });
    }
    getKvmEntry(name, env) {
        return __awaiter(this, void 0, void 0, function* () {
            var url = this.apigeeBaseUrl +
                "environments/" +
                env +
                "/keyvaluemaps/hpcorp-ob-paladin-kvm/entries/" +
                name;
            var response = this.makeApigeeApiRequest(url, "GET", "");
            return response;
        });
    }
    makeApigeeApiRequest(url, method, body, incomingHeader) {
        return __awaiter(this, void 0, void 0, function* () {
            let headers = {
                Authorization: "Bearer " + this.accessToken,
            };
            if (incomingHeader) {
                headers = incomingHeader;
            }
            const options = {
                method: method,
                headers: headers,
                url: url,
                data: "",
            };
            if (method.toLowerCase() === "post") {
                options.data = body;
            }
            let response = {};
            try {
                if (method.toLowerCase() == "delete" || method.toLowerCase() == "get") {
                    response = yield request(options);
                }
                else {
                    response = yield axios(options);
                }
                if (response) {
                    this.log.info("Successfully got the response for " + url);
                }
            }
            catch (error) {
                this.log.error("Error while making call to: " + url + ". Error is: " + error);
                response.result = false;
            }
            return response;
        });
    }
}
exports.ApigeeApiService = ApigeeApiService;
